import { WebSocketServer } from 'ws';
import { nanoid } from 'nanoid';
import { WsProxyManager } from './manager.ts';
import { getLoginUser } from '@/middleware/auth.ts';
export const wsProxyManager = new WsProxyManager();

export const upgrade = async (request: any, socket: any, head: any) => {
  const req = request as any;
  const url = new URL(req.url, 'http://localhost');
  const id = url.searchParams.get('id');
  if (url.pathname === '/ws/proxy') {
    console.log('upgrade', request.url, id);
    wss.handleUpgrade(req, socket, head, (ws) => {
      // 这里手动触发 connection 事件
      // @ts-ignore
      wss.emit('connection', ws, req);
    });
    return true;
  }
  return false;
};
export const wss = new WebSocketServer({
  noServer: true,
  path: '/ws/proxy',
});

wss.on('connection', async (ws, req) => {
  console.log('connected', req.url);
  // const user = await getLoginUser(req);
  // if (!user) {
  //   ws.send(
  //     JSON.stringify({
  //       type: 'error',
  //       message: 'Invalid authorization',
  //     }),
  //   );
  //   ws.close();
  //   return;
  // }
  const url = new URL(req.url, 'http://localhost');
  const id = url?.searchParams?.get('id') || nanoid();
  const user = 'root';
  wsProxyManager.register(id, { user, ws });
  ws.send(
    JSON.stringify({
      type: 'connected',
      id,
    }),
  );
  ws.on('message', async (event: Buffer) => {
    const eventData = event.toString();
    if (!eventData) {
      return;
    }
    const data = JSON.parse(eventData);
    console.log('message', data);
  });
  ws.on('close', () => {
    console.log('ws closed');
    wsProxyManager.unregister(id, user);
  });
});

export class WssApp {
  wss: WebSocketServer;
  constructor() {
    this.wss = wss;
  }
  upgrade(request: any, socket: any, head: any) {
    return upgrade(request, socket, head);
  }
}
