import { IncomingMessage, ServerResponse } from 'http';
import { wsProxyManager } from './index.ts';

import { App } from '@kevisual/router';
import { log } from 'console';
import { logger } from '../logger.ts';

type ProxyOptions = {
  createNotFoundPage: (msg?: string) => any;
};
export const UserV1Proxy = async (req: IncomingMessage, res: ServerResponse, opts?: ProxyOptions) => {
  const { url } = req;
  const { pathname } = new URL(url || '', `http://localhost`);
  const [user, app, userAppKey] = pathname.split('/').slice(1);
  if (!user || !app || !userAppKey) {
    opts?.createNotFoundPage?.('应用未启动');
    return false;
  }
  const data = await App.handleRequest(req, res);
  logger.debug('data', data);
  const client = wsProxyManager.get(userAppKey, user);
  const ids = wsProxyManager.getIds();
  if (!client) {
    opts?.createNotFoundPage?.(`应用未启动, 未找到应用, ${userAppKey}, ${ids.join(',')}`);
    return false;
  }
  const value = await client.sendData(data);
  if (value) {
    res.writeHead(200, { 'Content-Type': 'application/json' });
    res.end(JSON.stringify(value));
    return true;
  }
  opts?.createNotFoundPage?.('应用未启动');
  return true;
};
